import os
import pkg_resources

from celery import Celery

from .loader import YamlLoader


def resolve_entrypoint(use: str) -> Celery:
    entrypoint = "celery_yaml.app"
    try:
        pkg, name = use.split("#")
    except ValueError:
        pkg, name = use, "main"
    try:
        scheme, pkg = pkg.split(":")
    except ValueError:
        scheme = "egg"
    if scheme != "egg":
        raise ValueError(f"{use}: unsupported scheme {scheme}")

    distribution = pkg_resources.get_distribution(pkg)
    runner = distribution.get_entry_info(entrypoint, name)
    return runner.resolve()


def includeme(config):
    settings = config.registry.settings
    app = resolve_entrypoint(settings["celery"].pop("use"))
    celeryconf = settings["celery"]
    if "CELERY_BROKER_URL" in os.environ:
        # override the browker url
        celeryconf["broker_url"] = os.environ["CELERY_BROKER_URL"]
    app.config_from_object(celeryconf)



"""Helper for celery."""
import logging
import os
import sys
from logging.config import dictConfig

import yaml
from celery import VERSION as celery_version
import celery.loaders.base
from celery import Celery

log = logging.getLogger(__name__)


if celery_version.major < 5:

    from celery.signals import user_preload_options

    @user_preload_options.connect
    def on_preload_parsed(options, **kwargs):
        config_path = options["yaml"]
        app = kwargs["app"]

        if config_path is None:
            print("You must provide the --yaml argument")
            sys.exit(-1)

        try:
            loader = YamlLoader(app, config=config_path)
            loader.read_configuration()
        except Exception as err:
            print(err)
            sys.exit(-1)


def add_yaml_option(app):

    help = "Celery configuration in a YAML file."
    if celery_version.major < 5:

        def add_preload_arguments(parser):
            parser.add_argument("--yaml", default=None, help=help)

        app.user_options["preload"].add(add_preload_arguments)

    else:

        from click import Option
        from celery import bootsteps

        app.user_options["preload"].add(Option(["--yaml"], required=True, help=help))

        class YamlBootstep(bootsteps.Step):
            def __init__(self, parent, yaml: str = "", **options):
                try:
                    loader = YamlLoader(app, config=yaml)
                    loader.read_configuration()
                    loader.import_default_modules()
                except Exception as err:
                    print(err)
                    sys.exit(-1)
                super().__init__(parent, **options)

        app.steps["worker"].add(YamlBootstep)


class YamlLoader(celery.loaders.base.BaseLoader):
    """Celery loader based on yaml file."""

    def __init__(
        self, app: Celery, config: str, *, configure_logging: bool = True
    ) -> None:
        self.app = app
        self.config_path = config
        self.configure_logging = configure_logging
        super(YamlLoader, self).__init__(app)

    def read_configuration(self) -> dict:
        """Override this method to configure the celery app."""
        with open(self.config_path, "r") as stream:
            _conf = yaml.safe_load(stream)
            if "CELERY_BROKER_URL" in os.environ:
                # override the browker url
                _conf["celery"]["broker_url"] = os.environ["CELERY_BROKER_URL"]

            self.app.config_from_object(_conf["celery"])
            if self.configure_logging and "logging" in _conf:
                dictConfig(_conf["logging"])

            if hasattr(self.app, "on_yaml_loaded"):
                getattr(self.app, "on_yaml_loaded")(_conf, config_path=self.config_path)
        return _conf["celery"]



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['celery_yaml']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0,<7.0', 'celery>=4']

extras_require = \
{'pyramid': ['pyramid>=1.9', 'plaster-yaml>=0.1.3,<0.2.0']}

setup_kwargs = {
    'name': 'celery-yaml',
    'version': '0.4.0',
    'description': 'Easy configuration for celery app using Yaml file',
    'long_description': '# Easy Configuration For Celery App Using a Yaml File\n\n`celery-yaml` is a library to inject a --yaml option to the `celery worker`\ncommand in order to inject its configuration.\n\nIt also handle help to configurate this application for Pyramid application.\n\n## Usage\n\n### With Celery 4\n\n```sh\ncelery worker -A my_application.module_containing_my.app --yaml development.yaml ...\n```\n\n### With Celery 5\n\n```sh\ncelery -A my_application.module_containing_my.app worker --yaml development.yaml ...\n```\n\n\nThis will configure the application `my_application` containing an application\n`app` in a submodule `module_containing_my`.\n\nThe celery app must register the `--yaml` using the `add_yaml_option` on the\napp instance, this way:\n\n```python\nfrom celery import Celery\nfrom celery_yaml import add_yaml_option\n\napp = Celery()\nadd_yaml_option(app)\n```\n\n### Yaml format\n\n```yaml\ncelery:\n  broker_url: \'amqp://guest:guest@localhost:5672//\'\n  result_backend: \'rpc://\'\n  imports:\n      - my_application.tasks\n  # see all settings in the celery docs: \n  # https://docs.celeryproject.org/en/stable/userguide/configuration.html\n\nlogging:\n  version: 1\n  # dictConfig format\n  # https://docs.python.org/3/library/logging.config.html#logging-config-dictschema\n```\n\n---\n**NOTE**\n\nThe broker_url can also be override by an environment variable `CELERY_BROKER_URL`\nto avoid password in the configuration file.\n---\n\n## Using Celery in a Pyramid App.\n\nThe extras "pyramid" must be added to install the extras depencencies.\n\n### With poetry\n\n```toml\n[tool.poetry.dependencies]\ncelery-yaml = { version = "^0.1.3", extras = ["pyramid"] }\n```\n\nThen some entry_points have to configure, such as:\n\n```toml\n[tool.poetry.plugins."paste.app_factory"]\nmain = "pyramid_app.wsgi:main"\n\n[tool.poetry.plugins."celery_yaml.app"]\nmain = "pyramid_app.backend:app"\n\n[tool.poetry.plugins."plaster.loader_factory"]\n"file+yaml" = "plaster_yaml:Loader"\n```\n\nthe `paste.app_factory` is used by `Pyramid` itself to build the WSGI\nsergivice but we add a `plaster.loader_factory` to configure the usage\nof a yaml file instead of an `ini` file to configure it.\n\nThen the `celery_yaml.app` is used by `celery-yaml` as an entrypoint to\nthe celery app.\n\n\nThen, in the configuration file,\n\n```yaml\ncelery: &celery\n  result_backend: \'rpc://\'\n  imports:\n      - pyramid_app.tasks\n\napp:\n  "use": "egg:pyramid_app"\n  "pyramid.includes": ["celery_yaml"]\n  "celery":\n    <<: *celery\n    "use": "egg:pyramid_app"\n\n```\n\n### More configuration\n\nif the celery app as a method `on_yaml_loaded` then the function\nis called with the data and the filepath in parameter.\nIt may be used to get some config.\n\nExample\n\n```python\nfrom celery import Celery as CeleryBase\n\n\nclass Celery(CeleryBase):\n\n    def on_yaml_loaded(self, data: Dict[str, Any], config_path: str):\n      ...\n\n```\n\n\n\n#### See full example in the examples directory:\n\n##### Pyramid with Celery 4\nhttps://github.com/mardiros/celery-yaml/tree/master/examples/pyramid-app\n\n##### Pyramid with Celery 5\n\nhttps://github.com/mardiros/celery-yaml/tree/master/examples/pyramid-app5\n',
    'author': 'Guillaume Gauvrit',
    'author_email': 'guillaume@gauvr.it',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
